<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV101-56-g411a2eed:2023-07-24#


require_once(dirname(__FILE__)."/UniModul.php");


class UniAdapter {

	var $data=array();
	var $external = true;
	var $version = '1.0';

	protected $uniModul;


	public function __construct($uniModulName = null, $subMethod = null) {
		try {
			BeginUniErr();

			$uniFact = new UniModulFactory();
			$this->configInfo = $uniFact->getConfigInfo($uniModulName, $this->getLanguage(), $subMethod);

			$configSetting = $this->getConfigData($this->configInfo, $uniModulName, $subMethod);
			$this->configSetting = $configSetting;
			$this->uniModul = $uniFact->createUniModul($uniModulName, $configSetting, $subMethod);

			EndUniErr();
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

	function build_payment_info($data){

	}

	function pspRedirect($order_data) {
		try {
			BeginUniErr();

			global $xtLink,$filter,$order;

			// test callback
			//$url = $xtLink->_link(array('page'=>'callback', 'paction'=>'pti_testmodul'));
			//return $url;


			$orderToPayInfo = $this->getOrderToPayInfo();
			$prePayGWInfo = $this->uniModul->queryPrePayGWInfo($orderToPayInfo);
			if (!$prePayGWInfo->isPossible) {
				echo "This payment method cannot be used";  // nemelo by se sem vubec dostat, protoze se metoda ani nenabidne
				ResetUniErr();
				exit();
			}
			$redirectAction = $this->uniModul->gatewayOrderRedirectAction($orderToPayInfo);

			if ($redirectAction->orderReplyStatus != null) {   // okamzita odpoved muze byt i zaroven s redirektem - napr. Cetelem
				$frontend_redir = $redirectAction->redirectUrl==null && $redirectAction->redirectForm==null;
				//$this->procesReplyStatus($redirectAction->orderReplyStatus, $frontend_redir);
			}
			if ($redirectAction->redirectForm != null) {
				$this->uniModul->formRedirect($redirectAction->redirectForm);
				ResetUniErr();
				exit();
			} else if ($redirectAction->redirectUrl != null) {
				return EndUniErr($redirectAction->redirectUrl);
			}

			EndUniErr();
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

	function pspSuccess() {
		return true;
	}



	public function processCallback() {
		try {
			BeginUniErr();

			$orderReplyStatus = $this->uniModul->gatewayReceiveReply($this->getLanguage());
			$this->processReplyStatus($orderReplyStatus, true);

			EndUniErr();
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

	private function processReplyStatus($orderReplyStatus, $frontendRedir) {
		global $xtLink;

		BeginUniErr(E_UNIERR_DEFAULT & ~E_NOTICE);
		$order = new order($orderReplyStatus->shopPairingInfo, -1);
		EndUniErr();

		$oldStatus = $order->order_data['orders_status_id'];
		$defaultStatus = constant(strtoupper('pti_' . $this->uniModul->name . '_ORDER_STATUS_NEW')) ?? _STORE_DEFAULT_ORDER_STATUS;

		$cfg = $this->configSetting->configData;
		switch ($orderReplyStatus->orderStatus) {
			case OrderStatus::$successful:
				$newState = $cfg['orderStatusSuccessfull'];
				$canChangeState = in_array($oldStatus, array($defaultStatus, $cfg['orderStatusPending'], $cfg['orderStatusFailed']));
				$sendMail = true;
				$guiOk = true;
				break;
			case OrderStatus::$pending:
			case OrderStatus::$initiated:  // ? co jineho s takovou odpovedi
				$newState = $cfg['orderStatusPending'];
				$canChangeState = in_array($oldStatus, array($defaultStatus, $cfg['orderStatusFailed']));
				$sendMail = false;
				$guiOk = true;
				break;
			case OrderStatus::$failedFinal:
			case OrderStatus::$failedRetriable;
				$newState = $cfg['orderStatusFailed'];
				$canChangeState = in_array($oldStatus, array($defaultStatus, $cfg['orderStatusPending']));
				$sendMail = false;
				$guiOk = false;
				break;
			case OrderStatus::$invalidReply;
			default:
				$newState = null;
				$canChangeState = false;
				$sendMail = false;
				$guiOk = false;
				break;
		}

		if ($newState != null) {
			if ($canChangeState) {
				$this->uniModul->logger->writeLog("ADAPTER: Updating shop order number:" . $orderReplyStatus->shopOrderNumber . ", id:" . $orderReplyStatus->shopPairingInfo.", old order status ".$oldStatus.", new order status ".$newState);
				BeginUniErr(E_UNIERR_DEFAULT & ~E_NOTICE);
				$order->_updateOrderStatus($newState, /*comment*/ '', $sendMail, /*send_comments*/ 'true', $frontendRedir ? 'user':'IPN');
				EndUniErr();
			} else {
				$this->uniModul->logger->writeLog("ADAPTER: Not updating shop order number:" . $orderReplyStatus->shopOrderNumber . ", id:" . $orderReplyStatus->shopPairingInfo.", old order status ".$oldStatus.", new order status ".$newState);
			}
		}

		if ($frontendRedir) {
			if ($guiOk) {
				$redirUrl = $xtLink->_link(array('page'=>'checkout', 'paction'=>'payment_process', 'conn'=>'SSL', ));
			} else {
				$redirUrl = $xtLink->_link(array('page'=>'checkout', 'paction'=>'payment', 'conn'=>'SSL' , 'params'=>'unimodul_msg='.TEXT_PTI_PAYMENT_FAILED));
			}
			ResetUniErr();
			$xtLink->_redirect($redirUrl);
		}
	}


	protected function getConfigData($configInfo, $uniModulName, $subMethod) {
		global $xtLink;

		$configData = array();
		$configFields = $configInfo->configFields;
		foreach ($configFields as $configField) {
			if ($configField->type == ConfigFieldType::$subMethodsSelection || $configField->type == ConfigFieldType::$orderStatusMultiple) {
				$configData[$configField->name] = null;
			} else {
				$configData[$configField->name] = constant(strtoupper('pti_' . $uniModulName . '_' . $configField->name));
			}
		}

		$uniModulConfig = new UniModulConfig();
		$uniModulConfig->mysql_server = _SYSTEM_DATABASE_HOST;
		$uniModulConfig->mysql_dbname = _SYSTEM_DATABASE_DATABASE;
		$uniModulConfig->mysql_login = _SYSTEM_DATABASE_USER;
		$uniModulConfig->mysql_password = _SYSTEM_DATABASE_PWD;
		$uniModulConfig->uniModulDirUrl = str_replace('index.php?page=callback&amp;', '', $xtLink->_link(array('page'=>'callback', 'conn'=>'SSL' ))) . '/UniModul/';

		$uniModulConfig->adapterName = 'XtCommerce';
		$cfgs = create_initialize_object('ConfigSetting', array('configData'=>$configData, 'uniModulConfig'=>$uniModulConfig));
		return $cfgs;
	}


	protected function getLanguage() {
		global $language;
		return $language->code;

	}


	protected function getOrderToPayInfo() {
		global $order, $xtLink, $filter, $db, $countries;

		$orderToPayInfo = new OrderToPayInfo();

		$orderToPayInfo->shopPairingInfo = $order->oID;
		$orderToPayInfo->shopOrderNumber = $order->order_data['orders_id'];

		$replyUrl = $xtLink->_link(array('page'=>'callback', 'paction'=>'pti_'.strtolower($this->uniModul->name)));
		$orderToPayInfo->replyUrl = $replyUrl;

		$orderToPayInfo->amount = $order->order_total['total']['plain'];
		$orderToPayInfo->currency = $order->order_data['currency_code'];
		$orderToPayInfo->language = $this->getLanguage();
		$orderToPayInfo->description = _STORE_NAME;

		$customerData = new CustomerData();
		$customerData->first_name = $order->order_data['billing_firstname'];
		$customerData->last_name = $order->order_data['billing_lastname'];
		$customerData->email = $order->order_data['customers_email_address'];
		$customerData->phone = $order->order_data['billing_phone'];
		$customerData->street = $order->order_data['billing_street_address'];
		$customerData->city = $order->order_data['billing_city'];
		$customerData->post_code = $order->order_data['billing_postcode'];
		$customerData->country = $order->order_data['billing_country_code'];
		$orderToPayInfo->customerData = $customerData;

		return $orderToPayInfo;
	}

}
